//------------------------------------------------------------------------------------------------------------------------
// Class       : BP_TurokLongHunter
// Author      : BehemothProgrammer
// Description : Longhunter used the same as a normal enemy
// OnTrigger   : Nothing
//
// Parameters  : 
//------------------------------------------------------------------------------------------------------------------------

enum BPLongHunterState
{
    BP_LHS_IDLE = 0,
    BP_LHS_OBSERVING,
    BP_LHS_ATTACK,
    BP_LHS_DEAD
}

enum BPLongHunterStageInfo
{
    BP_LHSI_HEALTHPERCNT   = 0,
    BP_LHSI_SPEED,
    BP_LHSI_EVADETIME,
    BP_LHSI_EVADESKIPS,
    BP_LHSI_NOEVADETIME,
    BP_LHSI_FLINCHPERCNT,
    BP_LHSI_RUNTIME,
    BP_LHSI_GUNFIRECOUNT
}

// health %, speed, evade time, evade skips, no evade time, flinch %, run time before attack, gun fire total
const array<float> g_BPLonghuntStage_1 = { 75.0f, 1.0f,  1.0f, 4.0f, 4.0f, 20.0f, 2.0f,  2.0f };
const array<float> g_BPLonghuntStage_2 = { 50.0f, 0.95f, 2.0f, 3.0f, 5.0f, 12.0f, 1.75f, 2.0f };
const array<float> g_BPLonghuntStage_3 = { 25.0f, 0.9f,  2.0f, 2.0f, 6.0f, 6.0f,  1.5f,  3.0f };
const array<float> g_BPLonghuntStage_4 = { 0.0f,  0.85f, 3.0f, 1.0f, 7.0f, 4.0f,  1.0f,  3.0f };

const array<const array<float>@> g_BPLonghuntStages =
{
    @g_BPLonghuntStage_1,
    @g_BPLonghuntStage_2,
    @g_BPLonghuntStage_3,
    @g_BPLonghuntStage_4
};

const array<int> g_BPlonghuntMeleeAnims = { anim_aiMelee1, anim_aiMelee3, anim_longHunterTaunt };
const array<int> g_BPlonghuntMeleeWeights = { 50, 50, 5 };
const array<float> g_BPlonghuntMeleeSpeed = { 2.5, 2.0f, 3.0f };

const array<int> g_BPlonghuntRangeAnims = { anim_aiMelee4, anim_aiMelee5, anim_aiMelee6 };
const array<int> g_BPlonghuntRangeWeights = { 50, 15, 35 };
const array<float> g_BPlonghuntRangeSpeed = { 4.0f, 1.14f, 4.0f };

const array<int> g_BPlonghuntWalkAnims = { anim_aiMelee2, anim_aiMelee4 };
const array<int> g_BPlonghuntWalkWeights = { 50, 25 };
const array<float> g_BPlonghuntWalkSpeed = { 2.667f, 4.0f };

final class BP_TurokLongHunter : TurokEnemy
{
    int m_state;
    int m_stage;
    int m_rangeAttackLoops;
    int m_standingTauntTime;
    float m_runTime;
    float m_goalDistSq;
    float m_goalDist;
    float m_evadeTime;
    int m_evadeActive;
    int m_evadeCount;
    float m_startHealth;
    float m_animSpeed;
    kVec3 m_vGoalOrigin;
    kAngle m_desiredYaw;
    kAngle m_lookAtYaw;
    kAngle m_randomYaw;
    const array<float> @m_pStageMode;
	//------------------------------------------------------------------------------------------------------------------------
    BP_TurokLongHunter(kActor @actor)
    {
        m_goalDistSq = 0;
        m_goalDist = 0;
        m_desiredYaw = 0;
        m_lookAtYaw = 0;
        m_randomYaw = 0;
        m_runTime = 0;
        m_evadeTime = 0;
        m_evadeActive = 0;
        m_animSpeed = 1;
        m_rangeAttackLoops = 0;
        m_stage = 0;
        m_startHealth = 1;
        m_evadeCount = 0;
        m_standingTauntTime = 0;
        @m_pStageMode = g_BPLonghuntStages[0];
        m_state = BP_LHS_ATTACK;
        
        super(actor);
    }
	//------------------------------------------------------------------------------------------------------------------------
    void SwooshTrail2(kActor @instigator, const float w, const float x, const float y, const float z)
    {
        self.RenderModel().AddTrailEffect("Trail_LongFootEvade", 3);
        self.RenderModel().AddTrailEffect("Trail_LongFootEvade", 8);
    }
	//------------------------------------------------------------------------------------------------------------------------
    void SwooshTrail3(kActor @instigator, const float w, const float x, const float y, const float z)
    {
        self.RenderModel().AddTrailEffect("Trail_LongHand", 12);
        self.RenderModel().AddTrailEffect("Trail_LongHand", 16);
    }
	//------------------------------------------------------------------------------------------------------------------------
    void SwooshRightFoot(kActor @instigator, const float w, const float x, const float y, const float z)
    {
        self.RenderModel().AddTrailEffect("Trail_LongFoot", 8);
    }
	//------------------------------------------------------------------------------------------------------------------------
    void SwooshRightHand(kActor @instigator, const float w, const float x, const float y, const float z)
    {
        self.RenderModel().AddTrailEffect("Trail_LongHand", 18);
    }
	//------------------------------------------------------------------------------------------------------------------------
    void PickAnimation(const array<int> &anims, const array<int> &weights, const array<float> &speeds)
    {
        if(anims.length() != weights.length())
        {
            return;
        }
        
        int randomSum = 0;
        uint i;
        int count = 0;
        array<int> rndArray;
        array<int> animArray;
        array<float> animSpeed;
        
        for(i = 0; i < weights.length(); i++)
        {
            if(self.AnimState().CheckAnimID(anims[i]))
            {
                rndArray.insertLast(randomSum);
                animArray.insertLast(anims[i]);
                animSpeed.insertLast(speeds[i]);
                randomSum += weights[i];
                count++;
            }
        }
        
        if(randomSum == 0 || count == 0)
        {
            return;
        }
        
        int r = Math::RandMax(randomSum);
        
        for(i = count-1; i >= 0; i--)
        {
            if(rndArray[i] <= r)
            {
                break;
            }
        }
        
        BlendAnimation(animArray[i], animSpeed[i], 8.0f, false);
    }
	//------------------------------------------------------------------------------------------------------------------------
    void BlendAnimation(const int anim, const float speed, const float blend, const bool bStoppedOnly = true)
    {
        if(bStoppedOnly && !self.AnimState().Stopped())
        {
            return;
        }
        else if(!bStoppedOnly && self.AnimState().PlayingID() == anim)
        {
            return;
        }
        
        if(self.AnimState().PlayingID() == anim)
        {
            self.AnimState().Set(anim, speed, ANF_ROOTMOTION);
            return;
        }
        
        self.AnimState().Blend(anim, speed*m_animSpeed, blend*m_animSpeed, ANF_ROOTMOTION);
    }
	//------------------------------------------------------------------------------------------------------------------------
    bool IsRunning()
    {
        switch(self.AnimState().PlayingID())
        {
        case anim_aiRunning:
        case anim_aiTurn_L_Run:
        case anim_aiTurn_R_Run:
        case anim_aiTurn_B_Run:
            return true;
        }
        
        return false;
    }
	//------------------------------------------------------------------------------------------------------------------------
    bool IsWalking()
    {
        switch(self.AnimState().PlayingID())
        {
        case anim_aiWalking:
        case anim_aiTurn_L_Walk:
        case anim_aiTurn_R_Walk:
        case anim_aiTurn_B_Walk:
            return true;
        }
        
        return false;
    }
	//------------------------------------------------------------------------------------------------------------------------
    void CheckForEvade()
    {
        bool bCanEvade = false;
        
        m_evadeTime -= GAME_DELTA_TIME;
        
        if(m_evadeTime < 0)
        {
            // swap evade states
            m_evadeActive ^= 1;
            
            // reset evade/non-evade time
            if(m_evadeActive == 1)
            {
                m_evadeTime = m_pStageMode[BP_LHSI_EVADETIME];
            }
            else
            {
                m_evadeTime = m_pStageMode[BP_LHSI_NOEVADETIME];
            }
        }
        
        // can we evade in this mode?
        switch(self.AnimState().PlayingID())
        {
        case anim_longHunterTaunt:
        case anim_aiTurn_L_Run:
        case anim_aiTurn_R_Run:
        case anim_aiTurn_L_Stand:
        case anim_aiTurn_L_Walk:
        case anim_aiTurn_R_Stand:
        case anim_aiTurn_R_Walk:
        case anim_aiTurn_B_Stand:
        case anim_aiTurn_B_Walk:
        case anim_aiTurn_B_Run:
        case anim_aiStanding:
        case anim_aiWalking:
        case anim_aiRunning:
            bCanEvade = true;
            break;
        }
        
        if(!bCanEvade)
        {
            return;
        }
        
        if(m_evadeActive == 1 && ++m_evadeCount >= int(m_pStageMode[BP_LHSI_EVADESKIPS]) &&
            InPlayerProjectilePath(Math::Deg2Rad(10.0f)))
        {
            m_evadeCount = 0;
            
            if(m_lookAtYaw >= 0.0f)
            {
                BlendAnimation(anim_aiDodgeRight, 4.0f, 8.0f, false);
            }
            else
            {
                BlendAnimation(anim_aiDodgeLeft, 4.0f, 8.0f, false);
            }
        }
    }
	//------------------------------------------------------------------------------------------------------------------------
    bool CheckForWalkRunAttack()
    {
        if(m_state != BP_LHS_ATTACK)
        {
            return false;
        }
        
        bool bWalking = IsWalking();
        bool bRunning = IsRunning();
        int anim = self.AnimState().PlayingID();
        
        if(bWalking)
        {
            m_runTime += (GAME_DELTA_TIME*0.7f)*2;
        }
        else if(bRunning)
        {
            m_runTime += (GAME_DELTA_TIME*0.7f);
        }
        
        if(bWalking)
        {
            kVec3 vDir = (Player.Actor().Origin() - self.Origin()).Normalize();
            kVec3 vMoveDir = Player.Actor().Movement();
            
            if(vMoveDir.Dot(vDir) >= (0.2f*GAME_SCALE))
            {
                bWalking = false;
                bRunning = true;
            }
        }
        
        // been walking/running too long?
        if(m_runTime > m_pStageMode[BP_LHSI_RUNTIME])
        {
            m_runTime = 0;
            
            if (Math::RandMax(100) >= 80)
            {
				self.PlaySound("sounds/shaders/longhunter_taunt_1.ksnd");
            }
            
            if(bWalking)
            {
                PickAnimation(g_BPlonghuntWalkAnims, g_BPlonghuntWalkWeights, g_BPlonghuntWalkSpeed);
            }
            else if(bRunning)
            {
                PickAnimation(g_BPlonghuntRangeAnims, g_BPlonghuntRangeWeights, g_BPlonghuntRangeSpeed);
            }
        }
        
        return (self.AnimState().PlayingID() != anim);
    }
	//------------------------------------------------------------------------------------------------------------------------
    void GetTargetPoint()
    {
        float dist;
        float x, y;
        kVec3 dir;
        kVec3 org;
        
        if(self.AnimState().PlayingID() == anim_aiMelee6)
        {
            m_desiredYaw = 0;
            m_lookAtYaw = 0;
            return;
        }
        
        org = Player.Actor().Origin();
        
        dist = (Math::Sqrt(self.DistanceToPoint(org)) - (15*GAME_SCALE)) * 2.0f;
        if(dist > (80*GAME_SCALE))
        {
            dist = (80*GAME_SCALE);
        }
        else if(dist < 0)
        {
            dist = 0;
        }
        
        dir = (org - self.Origin()).Normalize();
        
        x = dir.y;
        y = dir.x;
        
        m_vGoalOrigin.x = Math::Sin(m_randomYaw) * x * dist + org.x;
        m_vGoalOrigin.y = Math::Cos(m_randomYaw) * y * dist + org.y;
        m_vGoalOrigin.z = self.Origin().z;
        
        m_randomYaw += Math::Deg2Rad(5.0f) * 0.5f;
        
        m_goalDistSq = self.DistanceToPoint(org);
        m_goalDist = Math::Sqrt(m_goalDistSq);
        
        m_desiredYaw = self.GetAvoidanceAngle(m_vGoalOrigin, 1.5f) - self.Yaw();
        m_lookAtYaw = self.GetTurnYaw(Player.Actor().Origin());
    }
	//------------------------------------------------------------------------------------------------------------------------
    void StandTurn()
    {
        bool bInterrupt = !(self.AnimState().PlayingID() == anim_aiStanding);
        
        if(self.AnimState().PlayingID() != anim_aiTurn_B_Stand)
        {
            if(m_lookAtYaw >= Math::Deg2Rad(-100.0f))
            {
                if(m_lookAtYaw > Math::Deg2Rad(-15.0f))
                {
                    if(m_lookAtYaw > Math::Deg2Rad(15.0f))
                    {
                        BlendAnimation(anim_aiTurn_R_Stand, 4.0f, 8.0f, bInterrupt);
                    }
                }
                else
                {
                    BlendAnimation(anim_aiTurn_L_Stand, 4.0f, 8.0f, bInterrupt);
                }
            }
            else
            {
                BlendAnimation(anim_aiTurn_B_Stand, 4.0f, 8.0f, bInterrupt);
            }
        }
        
        if(Math::Fabs(m_lookAtYaw) <= Math::Deg2Rad(8.0f))
        {
            BlendAnimation(anim_aiStanding, 4.0f, 8.0f, false);
        }
    }
	//------------------------------------------------------------------------------------------------------------------------
    void WalkTurn()
    {
        bool bInterrupt = !(self.AnimState().PlayingID() == anim_aiStanding);
        
        if(self.AnimState().PlayingID() != anim_aiTurn_B_Walk)
        {
            TurnAngles(Math::Deg2Rad(6.0f), m_desiredYaw);
            
            if(m_desiredYaw >= Math::Deg2Rad(-150.0f))
            {
                if(m_desiredYaw > Math::Deg2Rad(-20.0f))
                {
                    if(m_desiredYaw > Math::Deg2Rad(20.0f))
                    {
                        BlendAnimation(anim_aiTurn_R_Walk, 4.0f, 8.0f, bInterrupt);
                    }
                }
                else
                {
                    BlendAnimation(anim_aiTurn_L_Walk, 4.0f, 8.0f, bInterrupt);
                }
            }
            else
            {
                BlendAnimation(anim_aiTurn_B_Walk, 4.0f, 8.0f, bInterrupt);
            }
        }
        
        if(Math::Fabs(m_desiredYaw) <= Math::Deg2Rad(30.0f))
        {
            BlendAnimation(anim_aiWalking, 4.0f, 8.0f);
        }
    }
	//------------------------------------------------------------------------------------------------------------------------
    void RunTurn()
    {
        bool bInterrupt = !(self.AnimState().PlayingID() == anim_aiStanding);
        
        if(self.AnimState().PlayingID() != anim_aiTurn_B_Run)
        {
            TurnAngles(Math::Deg2Rad(12.0f), m_desiredYaw);
            
            if(m_desiredYaw >= Math::Deg2Rad(-150.0f))
            {
                if(m_desiredYaw > Math::Deg2Rad(-25.0f))
                {
                    if(m_desiredYaw > Math::Deg2Rad(25.0f))
                    {
                        BlendAnimation(anim_aiTurn_R_Run, 5.0f, 8.0f, bInterrupt);
                    }
                }
                else
                {
                    BlendAnimation(anim_aiTurn_L_Run, 5.0f, 8.0f, bInterrupt);
                }
            }
            else
            {
                BlendAnimation(anim_aiTurn_B_Run, 5.0f, 8.0f, bInterrupt);
            }
        }
        
        if(Math::Fabs(m_desiredYaw) <= Math::Deg2Rad(35.0f))
        {
            BlendAnimation(anim_aiRunning, 5.0f, 8.0f);
        }
    }
	//------------------------------------------------------------------------------------------------------------------------
    bool IsPlayerLocked()
    {
        return (Player.Actor().Health() > 0 && Player.Locked());
    }
	//------------------------------------------------------------------------------------------------------------------------
    void DoLockedTaunt()
    {
        if(!Player.Locked() || m_state == BP_LHS_OBSERVING)
        {
            return;
        }
        --m_standingTauntTime;
        if(m_standingTauntTime <= 0)
        {
			BlendAnimation(anim_longHunterTaunt, 4.0f, 8.0f, false);
			self.PlaySound("sounds/shaders/longhunter_taunt_1.ksnd");
			m_standingTauntTime = 250 + Math::RandMax(111);
        }
        else
        {
            BlendAnimation(anim_aiStanding, 4.0f, 8.0f, false);
        }
    }
	//------------------------------------------------------------------------------------------------------------------------
    void DoAttacks()
    {
        switch(self.AnimState().PlayingID())
        {
        case anim_aiTurn_L_Run:
        case anim_aiTurn_R_Run:
        case anim_aiTurn_B_Run:
            return;
        }
        
        // don't attack during cinematics unless the player is dead
        if(IsPlayerLocked())
        {
            DoLockedTaunt();
            return;
        }
        
        if(m_goalDist <= (12*GAME_SCALE) && Math::Fabs(m_desiredYaw) <= Math::Deg2Rad(15.0f))
        {
            PickAnimation(g_BPlonghuntMeleeAnims, g_BPlonghuntMeleeWeights, g_BPlonghuntMeleeSpeed);
        }
        else if(CheckForWalkRunAttack())
        {
            m_rangeAttackLoops = int(m_pStageMode[BP_LHSI_GUNFIRECOUNT]);
        }
    }
	//------------------------------------------------------------------------------------------------------------------------
    void Chase()
    {
        float dist = (6*GAME_SCALE);
        
        // don't chase toward player during cinematics unless he is dead
        if(IsPlayerLocked() && m_state != BP_LHS_OBSERVING)
        {
            DoLockedTaunt();
            return;
        }
        
        if(m_goalDist > (20*GAME_SCALE))
        {
            RunTurn();
        }
        else if(m_goalDist > dist)
        {
            WalkTurn();
        }
        
        if(m_goalDist <= dist)
        {
            StandTurn();
        }
        
        if(m_state == BP_LHS_ATTACK)
        {
            DoAttacks();
        }
        
        if(self.AnimState().Stopped())
        {
            self.AnimState().Blend(anim_aiStanding, 4.0f, 8.0f, ANF_ROOTMOTION);
        }
    }
	//------------------------------------------------------------------------------------------------------------------------
    void LoopAttack(const int anim, const float speed)
    {
        if(self.AnimState().Stopped())
        {
            m_rangeAttackLoops--;
        }
        else
        {
            TurnAngles(Math::Deg2Rad(12.0f), m_lookAtYaw);
            return;
        }
        
        if(m_rangeAttackLoops > 0)
        {
            self.AnimState().Blend(anim, speed, 4.0f, ANF_ROOTMOTION);
            TurnAngles(Math::Deg2Rad(20.0f), m_lookAtYaw);
        }
        else
        {
            DoAttacks();
        }
        
        if(self.AnimState().Stopped())
        {
            Chase();
        }
    }
	//------------------------------------------------------------------------------------------------------------------------
    void UpdateStage()
    {
        float healthPercnt = float(self.Health()*100) / m_startHealth;
        
        if(healthPercnt < m_pStageMode[BP_LHSI_HEALTHPERCNT])
        {
            m_stage++;
            if(m_stage >= int(g_BPLonghuntStages.length()))
            {
                m_stage = int(g_BPLonghuntStages.length()-1);
            }
            
            @m_pStageMode = g_BPLonghuntStages[m_stage];
            m_animSpeed = m_pStageMode[BP_LHSI_SPEED];
        }
    }
	//------------------------------------------------------------------------------------------------------------------------
    void OnTick()
    {
        switch(m_state)
        {
			case BP_LHS_ATTACK:
				GetTargetPoint();
				CheckForEvade();
				break;

			case BP_LHS_DEAD:
				return;
        }
        
        UpdateStage();
        
        switch(self.AnimState().PlayingID())
        {
        case anim_aiTurn_L_Run:
        case anim_aiTurn_R_Run:
            if(Math::Fabs(m_desiredYaw) <= Math::Deg2Rad(15.0f))
            {
                BlendAnimation(anim_aiRunning, 4.0f, 8.0f, false);
                break;
            }
            // fall through
        case anim_aiTurn_L_Stand:
        case anim_aiTurn_L_Walk:
        case anim_aiTurn_R_Stand:
        case anim_aiTurn_R_Walk:
        case anim_aiTurn_B_Stand:
        case anim_aiTurn_B_Walk:
        case anim_aiTurn_B_Run:
        case anim_aiStanding:
        case anim_aiWalking:
        case anim_aiRunning:
            Chase();
            break;
            
        case anim_aiDodgeRight:
        case anim_aiDodgeLeft:
        case anim_aiMelee1:
        case anim_aiMelee2:
        case anim_aiMelee3:
        case anim_longHunterTaunt:
            if(self.AnimState().Stopped())
            {
                DoAttacks();
                
                if(self.AnimState().Stopped())
                {
                    Chase();
                }
            }
            else
            {
                float max = m_goalDist / (4*GAME_SCALE);
                Math::Clamp(max, 2.0f, 50.0f);
                
                TurnAngles(Math::Deg2Rad(max), m_lookAtYaw);
            }
            break;
            
        case anim_aiMelee4:
            LoopAttack(anim_aiMelee4, 4.0f);
            break;
            
        case anim_aiMelee5:
            LoopAttack(anim_aiMelee5, 1.14f);
            break;
            
        case anim_aiMelee6:
            LoopAttack(anim_aiMelee6, 4.0f);
            break;
        }
    }
	//------------------------------------------------------------------------------------------------------------------------
    void OnBeginLevel()
    {
        m_startHealth = float(self.Health());
        self.SetTarget(Player.Actor().CastToActor());
        self.AnimState().Set(anim_aiStanding, 4.0f, ANF_ROOTMOTION);
		m_state = BP_LHS_ATTACK;
		self.SetHeadTrackTarget(Player.Actor().CastToActor());
    }
	//------------------------------------------------------------------------------------------------------------------------
    void OnDamage(kActor @instigator, kDictMem @damageDef, const int damage)
    {
        if(self.Health() <= 0)
        {
            return;
        }
        
        if (Math::RandMax(99) > 80)
        {
			self.PlaySound("sounds/shaders/longhunter_taunt_3.ksnd");
        }
    }
	//------------------------------------------------------------------------------------------------------------------------
    void OnDeath(kActor @killer, kDictMem @damageDef)
    {
        self.SetTarget(null);
        self.SetHeadTrackTarget(null);
        
        m_state = BP_LHS_DEAD;
		
		self.AnimState().Set(anim_aiDeathStand, 4.0f, ANF_ROOTMOTION);
		self.Flags() &= ~AF_SOLID;
		self.Flags() |= AF_DEAD;
		self.MarkPersistentBit(false);
    }
	//------------------------------------------------------------------------------------------------------------------------
	void OnRestore()
	{
		self.Remove();
	}
	//------------------------------------------------------------------------------------------------------------------------
};
