//------------------------------------------------------------------------------------------------------------------------
// Class       : BP_Counter
// Author      : BehemothProgrammer
// Description : A counter that increases each time it's triggered. Once it's limit is reached it can trigger actors and scripts.
//               It's current value is persistent by use of GameVariables with it's Counter ID.
// OnTrigger   : Increases counter value by 1. When the limit is reached the persistent bit is set and subsequent triggers do nothing.
//
// Parameters  : (7) Params 1 -128..127 				: Counter ID 0..127
//               (3) Max Regenerations -32768..32767	: Counter Limit
//               (0) Target TID -32768..32767 			: Actors TID to trigger when counter has reached it's limit. If is less than 1 this will not be used.
//               (2) Health -32768..32767				: Script to trigger when counter has reached it's limit. If is less than 1 this will not be used.
//------------------------------------------------------------------------------------------------------------------------

class BP_Counter : BP_ScriptObject
{
	int id;
	int limit;
	int triggerTID;
	int triggerScriptID;
	//------------------------------------------------------------------------------------------------------------------------
    BP_Counter(kActor @actor)
	{
		super(actor);
    }
	//------------------------------------------------------------------------------------------------------------------------
    void OnBeginLevel()
    {
		id = self.SpawnParams(7);
		limit = self.SpawnParams(3);
		triggerTID = self.SpawnParams(0);
		triggerScriptID = self.SpawnParams(2);
		
		self.Flags() &= ~AF_SOLID;
		self.Flags() |= AF_INVINCIBLE;
	}
	//------------------------------------------------------------------------------------------------------------------------
    void OnRestore()
    {
		self.Flags() |= AF_ACTIVATED;
    }
	//------------------------------------------------------------------------------------------------------------------------
	void OnActivate()
	{
		if (id < 0 || id >= MAX_COUNTERS)
			return;
		
		int value = Game::IncreaseCounter(id);
		if (value >= limit)
		{
			self.MarkPersistentBit(false);
			if (triggerTID > 0)
				World.TriggerActorsByTID(self, triggerTID);
			if (triggerScriptID > 0)
				Game.CallDelayedMapScript(triggerScriptID, self, 0.0f);
		}
		else
		{
			self.Flags() &= ~AF_ACTIVATED;
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
};
