//------------------------------------------------------------------------------------------------------------------------
// Class       : BP_FxSpawner
// Author      : BehemothProgrammer
// Description : Spawns an Fx with a delay and other flag options.
// OnTrigger   : Spawns the fx after delay. If OneShot flag is on then will not spawn again and is persistent.
//
// Parameters  : (3) Max Regenerations -32768..32767	: Flags
//               (2) Health -32768..32767				: Fx to spawn (ie. 12 = explosion)
//               (0) Target TID -32768..32767 			: Sound to play when fx spawns. If is less than 1 this will not be used. (ie. 66 = explosion_2)
//               Radius									: Delay before spawning fx
//------------------------------------------------------------------------------------------------------------------------

enum BPFxSpawnerFlags
{
	BP_FXSPAWNER_FLAG_ONESHOT = (1 << 0), //plays only once and sets persistent bit
	BP_FXSPAWNER_FLAG_SPAWNONSTART = (1 << 1) //spawns automatically on start
}

class BP_FxSpawner : BP_ScriptObject
{
	int flags;
	int fxID;
	int soundID;
	float delay;
	
	float delayTime;
	//------------------------------------------------------------------------------------------------------------------------
    BP_FxSpawner(kActor @actor)
    {
		super(actor);
		delayTime = -1.0f;
    }
	//------------------------------------------------------------------------------------------------------------------------
    void OnBeginLevel()
    {
		flags = self.SpawnParams(3);
		fxID = self.SpawnParams(2);
		soundID = self.SpawnParams(0);
		delay = Math::Maxf(self.Radius(), 0.0f);
		
		self.Flags() &= ~AF_SOLID;
		self.Flags() |= AF_INVINCIBLE;
		
		if (self.IsPersistentMarked())
		{
			self.Flags() |= AF_ACTIVATED;
		}
		else
		{
			if (flags & BP_FXSPAWNER_FLAG_SPAWNONSTART != 0)
				StartSpawnFx();
		}
    }
	//------------------------------------------------------------------------------------------------------------------------
    void OnTick() override
    {
		if (delayTime >= 0.0f && Time::Reached(delayTime))
		{
			delayTime = -1.0f;
			if (Game::HasFx(fxID))
			{
				self.PlaySoundWithLookupID(soundID);
				self.SpawnFx(Game::GetFx(fxID), Math::vecZero, self.Rotation());
			}
		}
    }
	//------------------------------------------------------------------------------------------------------------------------
	void OnActivate()
	{
		self.Flags() &= ~AF_ACTIVATED;
		
		if (delayTime < 0.0f)
		{
			StartSpawnFx();
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
	void StartSpawnFx()
	{
		delayTime = Time::FromNow(delay);
		if (flags & BP_FXSPAWNER_FLAG_ONESHOT != 0)
		{
			self.Flags() |= AF_ACTIVATED;
			self.MarkPersistentBit(false);
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
};
