//------------------------------------------------------------------------------------------------------------------------
// Class       : BP_HUDMessages
// Author      : BehemothProgrammer
// Description : Shows messages when triggered
// OnTrigger   : Shows the message. If OneShot flag is on then will not show again and is persistent.
//
// Parameters  : (7) Params 1 -128..127 				: Message Type
//               (6) Params 2 -128..127 				: hubID for SHOWKEYSLEFT type
//               (3) Max Regenerations -32768..32767	: Flags
//               Radius									: Text Line 1 (if < 0 then blank line)
//               WallRadius								: Text Line 2
//               Height									: Text Line 3
//               StepHeight/DeadHeight					: Text Line 4
//               (2) Health -32768..32767				: Text Line 5
//               (0) Target TID -32768..32767 			: Text Line 6
//------------------------------------------------------------------------------------------------------------------------

enum BPHUDMessageTypes
{
	BP_HUDMSGTYPE_NORMAL = 0, //Shows each line value as Localized Text
	BP_HUDMSGTYPE_SHOWKEYSLEFT = 1, //Shows the number of keys remaining for the hubID
	BP_HUDMSGTYPE_PREPEND1 = 2, //prepends line 6 value to line 1
	BP_HUDMSGTYPE_PREPEND2 = 3 //prepends line 6 counter index value to line 1
}

enum BPHUDMessageFlags
{
	BP_HUDMSG_FLAG_ONESHOT = (1 << 0) //plays only once and sets persistent bit
}

class BP_HUDMessages : BP_ScriptObject
{
	int msgType;
	int flags;
	int arg;
	array<int> TextIDs;
	int hubID;
	//------------------------------------------------------------------------------------------------------------------------
    BP_HUDMessages(kActor @actor)
    {
		super(actor);
    }
	//------------------------------------------------------------------------------------------------------------------------
    void OnBeginLevel()
    {
		msgType = int(self.SpawnParams(7));
		hubID = Math::Max(int(self.SpawnParams(6)), 0);
		flags = self.SpawnParams(3);
		TextIDs.insertLast(int(self.Radius()));
		TextIDs.insertLast(int(self.WallRadius()));
		TextIDs.insertLast(int(self.Height()));
		TextIDs.insertLast(int(self.StepHeight()));
		TextIDs.insertLast(self.SpawnParams(2));
		TextIDs.insertLast(self.SpawnParams(0));
		
		self.Flags() &= ~AF_SOLID;
		self.Flags() |= AF_INVINCIBLE;
    }
	//------------------------------------------------------------------------------------------------------------------------
	void OnRestore()
	{
		self.Flags() |= AF_ACTIVATED;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void OnActivate()
	{
		if (flags & BP_HUDMSG_FLAG_ONESHOT != 0)
		{
			self.MarkPersistentBit(false);
		}
		else
		{
			self.Flags() &= ~AF_ACTIVATED;
		}
		
		ShowMessages();
	}
	//------------------------------------------------------------------------------------------------------------------------
	void ShowMessages()
	{
		array<kStr> lines;
		switch (msgType)
		{
			case BP_HUDMSGTYPE_PREPEND1: //prepends line 6 value to line 1
			case BP_HUDMSGTYPE_PREPEND2: //prepends line 6 counter index value to line 1
			{
				for (uint i = 0; i < TextIDs.length(); i++)
				{
					if (i == 5)
					{
						lines[0] = "" + ((msgType == BP_HUDMSGTYPE_PREPEND2) ? Game::GetCounter(TextIDs[i]) : TextIDs[i]);
					}
					else
					{
						if (TextIDs[i] >= 0)
							lines.insertLast(Game.GetLocalizedText("$str_" + TextIDs[i]));
						else if (i == 0)
							lines.insertLast("");
					}
				}

				break;
			}
			case BP_HUDMSGTYPE_SHOWKEYSLEFT:
			{
				int nKeys, remainingKeys, keyBits;
				if (!Game.GetHubKeyInfo(hubID, nKeys, remainingKeys, keyBits))
					return;
				
				if (remainingKeys <= 0)
				{
					lines.insertLast(Game.GetLocalizedText("$str_005")); //this level found
					lines.insertLast(Game.GetLocalizedText("$str_004")); //all keys on
				}
				else if (remainingKeys == 1)
				{
					lines.insertLast(Game.GetLocalizedText("$str_003")); //on this level
					lines.insertLast(Game.GetLocalizedText("$str_002")); //1 Key to Find
				}
				else
				{
					lines.insertLast(Game.GetLocalizedText("$str_001")); //on this level
					lines.insertLast("" + remainingKeys + " " + Game.GetLocalizedText("$str_000")); //x Keys to Find
				}
				
				break;
			}
			default:
			{
				for (uint i = 0; i < TextIDs.length(); i++)
				{
					if (TextIDs[i] >= 0)
						lines.insertLast(Game.GetLocalizedText("$str_" + TextIDs[i]));
				}
				
				break;
			}
		}
		
		HUD::ShowMessage(lines, 120);
	}
	//------------------------------------------------------------------------------------------------------------------------
};
