//------------------------------------------------------------------------------------------------------------------------
// Class       : BP_PlaySound
// Author      : BehemothProgrammer
// Description : Plays a sound when triggered
// OnTrigger   : Plays the sound
//
// Parameters  : (3) Max Regenerations -32768..32767	: Flags
//               (0) Target TID -32768..32767			: Sound ID
//               Radius									: Delay until sound plays
//------------------------------------------------------------------------------------------------------------------------

enum BPPlaySoundFlags
{
	BP_PSF_REPEATABLE = (1 << 0),
	BP_PSF_LOOPING = (1 << 1)
}

enum BPPlaySoundStates
{
    BP_PSS_IDLE = 0,
	BP_PSS_DELAYING
}

class BP_PlaySound : BP_ScriptObject
{
	int flags;
	int soundID;
	float delay;
	//------------------------------------------------------------------------------------------------------------------------
    BP_PlaySound(kActor @actor)
    {
		super(actor);
		state = BP_PSS_IDLE;
    }
	//------------------------------------------------------------------------------------------------------------------------
    void OnBeginLevel()
    {
		flags = self.SpawnParams(3);
		soundID = self.SpawnParams(0);
		delay = self.Radius();
		
		SetState(BP_PSS_IDLE);
		
		self.Flags() &= ~AF_SOLID;
		self.Flags() |= AF_INVINCIBLE;
    }
	//------------------------------------------------------------------------------------------------------------------------
	void OnRestore()
	{
		self.Flags() |= AF_ACTIVATED;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void OnTick() override
	{
		switch (state)
		{
			case BP_PSS_IDLE:
			{
				if (flags & BP_PSF_LOOPING != 0 && self.IsPersistentMarked())
					PlaySound();
				break;
			}
			case BP_PSS_DELAYING:
			{
				if (Time::Reached(stateEndTime))
				{
					PlaySound();
					SetState(BP_PSS_IDLE);
				}
				break;
			}
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
	void OnActivate()
	{
		if (flags & BP_PSF_REPEATABLE != 0 && flags & BP_PSF_LOOPING == 0)
		{
			self.Flags() &= ~AF_ACTIVATED;
			if (state != BP_PSS_IDLE)
				return;
		}
		else
		{
			self.MarkPersistentBit(false);
		}
		
		if (delay > 0.0f)
			SetState(BP_PSS_DELAYING);
		else
			PlaySound();
	}
	//------------------------------------------------------------------------------------------------------------------------
	void OnStateChange() override
	{
		switch (state)
		{
			case BP_PSS_IDLE:
			{
				self.Flags() &= ~AF_ALWAYSACTIVE;
				break;
			}
			case BP_PSS_DELAYING:
			{
				self.Flags() |= AF_ALWAYSACTIVE;
				self.Flags() &= ~AF_DORMANT;
				stateEndTime = Time::FromNow(delay);
				break;
			}
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
	void PlaySound()
	{
		if (soundID > 0)
			self.PlaySoundWithLookupID(soundID);
	}
	//------------------------------------------------------------------------------------------------------------------------
};
