//------------------------------------------------------------------------------------------------------------------------
// Class       : BP_Map
// Author      : BehemothProgrammer
// Description : Map manager actor that should be spawned in via script only and never placed in the map itself with the editor.
// OnTrigger   : Nothing.
//
// Parameters  : 
//------------------------------------------------------------------------------------------------------------------------

int mapActorCount;
bool playerDiedOnMapEnd;

class BP_Map : BP_ScriptObject
{
	float playerKnifeAttackTime;
	kActor@ musicSource; //The musics actor audio source
	kStr musicPath; //Current music playing
	//------------------------------------------------------------------------------------------------------------------------
    BP_Map(kActor @actor)
    {
		super(actor);
		if (mapActorCount > 0)
		{
			self.Remove();
			return;
		}
		
		mapActorCount++;
		BP_MapStartSetup();

		//stop the music if not suppose to play in this mapID
		int mapID = GameVar::GetInt(GV_BP_KEY_MUSICMAP);
		if (mapID != -2 && mapID != Game.GetCurrentMapID())
		{
			Music::Stop();
		}
    }
	//------------------------------------------------------------------------------------------------------------------------
	void OnPostBeginLevel()
	{
		//Bug fix: After death animation the player faces the opposite direction
		if (playerDiedOnMapEnd)
			playerActor.Yaw() += Math::pi;
	}
	//------------------------------------------------------------------------------------------------------------------------
    void OnTick() override
    {
		GameVar::SetFloat(GV_BP_KEY_GAMETIME, Time::Game() + GAME_DELTA_TIME);
		MusicTick();
		Input::Tick();
		
		if (Input::AttackPress() && Player.CurrentWeapon() == TW_WEAPON_KNIFE && Time::Reached(playerKnifeAttackTime) &&
			!Player.Locked() && !GameVar::GetIntFlag(GV_BP_KEY_FLAGS, BP_SF_NOKNIFEINTERACTION))
		{
			playerKnifeAttackTime = Time::FromNow(0.7f);
			self.InteractActorsAtPosition(Player::MeleePos(), FUNC_PLAYERKNIFEATTACK);
		}
    }
	//------------------------------------------------------------------------------------------------------------------------
	void OnEndLevel()
	{
		mapActorCount = 0;
		playerDiedOnMapEnd = Actor::IsDead(playerActor);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void MusicTick()
	{
		musicPath = GameVar::GetString(GV_BP_KEY_MUSIC);
		if (!String::IsEmpty(musicPath)) //is playing
		{
			if (!Actor::Exists(musicSource))
			{
				@musicSource = Actor::Spawn(ACTOR_NAME_DUMMY);
			}
			else
			{
				if (Cinema::IsActive())
				{
					if (Actor::IsDead(playerActor))
					{
						musicSource.StopLoopingSounds();
					}
					else
					{
						musicSource.Origin() = Camera.origin;
						musicSource.PlaySound(musicPath);
					}
				}
				else
				{
					musicSource.Origin() = playerActor.Origin();
					musicSource.PlaySound(musicPath);
				}
			}
		}
		else
		{
			if (Actor::Exists(musicSource))
			{
				musicSource.StopLoopingSounds();
			}
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
	void PlayerKnifeAttack(kActor @actor, const float arg1, const float arg2, const float arg3, const float arg4)
	{
		if (!Actor::Exists(actor) || Actor::IsDead(actor) || actor.ImpactType() == IT_FORCEFIELD ||
			(actor.Flags() & AF_HIDDEN) != 0 || (actor.Flags() & AF_HOLDTRIGGERANIM) != 0)
			return;
		
		int knifeImpactType = Actor::GetDefInt(actor, DEF_KNIFEIMPACT, -1);
		if (knifeImpactType >= 0)
        {
			int frontDamage = Actor::GetDefInt(actor, DEF_KNIFEDAMAGE, 0);
			int backDamage = Actor::GetDefInt(actor, DEF_KNIFEBACKDAMAGE, 0);
			
			float radius = Player::MELEE_RADIUS;
			kVec3 meleePos = Player::MeleePos();
			float dist = actor.Origin().Distance(meleePos);
			int angle1 = int(Math::Rad2Deg(actor.Yaw())) % 360;
			int angle2 = int(Math::Rad2Deg(playerActor.Yaw())) % 360;
			if(angle1 < 0) angle1 += 360;
			if(angle2 < 0) angle2 += 360;
			int angDiff = (angle1 - angle2);
			bool inRange = false;
			int damage = 0;
			// check for backstabs
			if (dist <= ((radius * 0.5f) + actor.Radius()) && Math::Abs(angDiff) <= 40)
			{
				damage = backDamage;
			}
			// do regular direct damage
			else if (dist <= (radius + actor.Radius()))
			{
				damage = frontDamage;
			}
			
	        if (damage <= 0)
				return;
			
			kVec3 impactPos = kVec3(0.0f, 0.0f, meleePos.z);
			switch (knifeImpactType)
			{
				case IT_METAL:
					actor.PlaySound("sounds/shaders/tomahawk_impact_flesh.ksnd");
					actor.SpawnFx("fx/generic_90.kfx", kVec3(0.0f, 0.0f, meleePos.z)); //sparks
					break;
				case IT_FLESH_HUMAN:
					actor.PlaySound("sounds/shaders/tomahawk_impact_flesh.ksnd");
					actor.SpawnFx("fx/generic_244.kfx", impactPos); //5 red bloods spit out
					break;
				case IT_FLESH_CREATURE:
					actor.PlaySound("sounds/shaders/tomahawk_impact_flesh.ksnd");
					actor.SpawnFx("fx/greenblood.kfx", impactPos);
					break;
				case IT_WOOD:
					actor.PlaySound("sounds/shaders/tomahawk_impact_flesh.ksnd");
					actor.SpawnFx("fx/BP/impact_wood.kfx", impactPos);
					break;
			}
			
			int lastHealth = actor.Health();
			actor.InflictGenericDamage(playerActor, damage);
			
			if (actor.ScriptObject() !is null && actor.ScriptObject().obj !is null)
			{
				TurokEnemy@ enemyObj = cast<TurokEnemy@>(actor.ScriptObject().obj);
				if (lastHealth > 0 && actor.Health() <= 0 && enemyObj !is null)
				{
					enemyObj.m_bMortallyWounded = true;
				}
			}
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
};
