//------------------------------------------------------------------------------------------------------------------------
// Class       : BP_ActorIterator
// Author      : BehemothProgrammer
// Description : Calls custom tick functions on each actor one at a time per tick. It is required that a onTick funcdef
//				 is assigned.
//------------------------------------------------------------------------------------------------------------------------

funcdef bool ACTORITERTICKFUNC(kActor@ actor); //return true to remove actor from actors array
funcdef void ACTORITEREMPTYFUNC();

final class BP_ActorIterator
{
	array<kActor@> actors;
	int maxPerTick;
	int index;
	ACTORITERTICKFUNC@ onTick;
	ACTORITEREMPTYFUNC@ onEmpty; //Called when actors length becames 0
	//------------------------------------------------------------------------------------------------------------------------
	BP_ActorIterator()
	{
		index = 0;
		maxPerTick = 1;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void Setup(const int perTick, ACTORITERTICKFUNC@ tickFunc, ACTORITEREMPTYFUNC@ emptyFunc = null)
	{
		maxPerTick = perTick;
		@onTick = tickFunc;
		@onEmpty = emptyFunc;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void AddActor(kActor@ actor)
	{
		actors.insertLast(actor);
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool IsEmpty()
	{
		return actors.length() == 0;
	}
	//------------------------------------------------------------------------------------------------------------------------
	uint ActorLength()
	{
		return actors.length();
	}
	//------------------------------------------------------------------------------------------------------------------------
	void CallOnEmpty()
	{
		if (onEmpty !is null)
			onEmpty();
	}
	//------------------------------------------------------------------------------------------------------------------------
	void Clear()
	{
		index = 0;
		uint startActorsLength = actors.length();
		if (startActorsLength > 0)
		{
			for (uint i = 0; i < startActorsLength; i++)
			{
				if (Actor::Exists(actors[i]))
				{
					actors[i].Remove();
				}
			}
			actors.resize(0);
			CallOnEmpty();
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
	void Tick()
	{
		int actorsTicked = 0;
		kActor@ actor;
		bool remove = false;
		int startActorsLength = int(actors.length());
		int maxActorsToTick = Math::Min(startActorsLength, maxPerTick);
		
		while (actors.length() > 0 && actorsTicked < maxActorsToTick)
		{
			@actor = actors[index];
			remove = onTick(actor);
			actorsTicked++;
			
			if (remove)
			{
				actors.removeAt(index);
				index = Array::NextIndex(actors.length(), index, 0);
			}
			else
			{
				index = Array::NextIndex(actors.length(), index);
			}
		}
		
		if (startActorsLength > 0 && actors.length() == 0)
			CallOnEmpty();
	}
	//------------------------------------------------------------------------------------------------------------------------
};
