#include "scripts/common.txt"

#include "BP_Addons/scripts/Utils/BP_Utils_Math.cpp"
#include "BP_Addons/scripts/Utils/BP_Utils_Array.cpp"
#include "BP_Addons/scripts/Utils/BP_Utils_Actor.cpp"
#include "BP_Addons/scripts/Utils/BP_Utils_Hud.cpp"
#include "BP_Addons/scripts/Utils/BP_Utils_Music.cpp"
#include "BP_Addons/scripts/Utils/BP_Utils_String.cpp"
#include "BP_Addons/scripts/Utils/BP_Utils_GameVars.cpp"
#include "BP_Addons/scripts/Utils/BP_Utils_Input.cpp"
#include "BP_Addons/scripts/Utils/BP_Utils_Cinema.cpp"
#include "BP_Addons/scripts/Utils/BP_Utils_Player.cpp"
#include "BP_Addons/scripts/Utils/BP_Utils_Game.cpp"
#include "BP_Addons/scripts/Utils/BP_Utils_Time.cpp"
#include "BP_Addons/scripts/Utils/BP_Utils_Inventory.cpp"

#include "BP_Addons/scripts/BP_ActorIterator.cpp"

const kStr GV_BP_KEY_FLAGS = "BP_Flags";
const kStr GV_BP_KEY_GAMETIME = "BP_GameTime";
const kStr GV_BP_KEY_MUSIC = "BP_Music";
const kStr GV_BP_KEY_MUSICMAP = "BP_MusicMap";
const kStr GV_BP_KEY_COUNTERS = "BP_Counters";
const kStr GV_BP_KEY_INVENTORY = "BP_Inventory";
const kStr GV_BP_KEY_PANELPIECES = "hubPanelPieces";

const kStr DEF_KNIFEIMPACT = "BP_KnifeImpact";
const kStr DEF_KNIFEDAMAGE = "BP_KnifeDamage";
const kStr DEF_KNIFEBACKDAMAGE = "BP_KnifeBackDamage";
const kStr DEF_SOLID = "BP_Solid";
const kStr DEF_DESTRUCTIBLE_FLAGS = "BP_DestructibleFlags";
const kStr DEF_DESTRUCTIBLE_ITEMCHANCE = "BP_DestructibleItemChance";
const kStr DEF_DAMAGE_EXPLODE = "bExplode";
const kStr DEF_STRING_HASH = "Hash";
const kStr DEF_STRING_DOLLAR = "Dollar";

const kStr FUNC_PLAYERKNIFEATTACK = "PlayerKnifeAttack";

const int MAX_COUNTERS = 128;
const int MAX_INVENTORY = 128;

const int SCRIPT_CINEMA_CAMERA_MOVE = 50000;
const int SCRIPT_CINEMA_USER_INTERRUPT = 50001;
const int SCRIPT_CINEMA_CAMERA_SHAKE = 50002;

const kStr ACTOR_NAME_DUMMY = "BP_Dummy";
const kStr ACTOR_NAME_DUMMYMODEL = "BP_DummyModel";
const kStr ACTOR_NAME_GAMEDEF = "BP_GameDef";
const kStr ACTOR_NAME_MANTISSTATUE = "BP_MantisStatue";
const kStr ACTOR_NAME_QUAKESOURCE = "BP_QuakeSource";

funcdef bool WAITFUNC();
funcdef void DOFUNC();

enum BPActorTypes
{
	//Gameplay
    BP_AT_POINT = 3000,
    BP_AT_EVENTLINK = 3001,
    BP_AT_COUNTER = 3002,
    BP_AT_ACTORSPAWNER = 3003,
    BP_AT_FXSPAWNER = 3004,
    BP_AT_TRIGGERAREA = 3005,
    BP_AT_SECTORMODIFIER = 3006,
    BP_AT_MESSENGER = 3007,
	
	//Misc
    BP_AT_1KEYPANEL = 3020,
    BP_AT_3KEYPANEL = 3021,
    BP_AT_LEVELPORTAL = 3022,
    BP_AT_PORTALGATE = 3023,
    BP_AT_KEYPLAQUE = 3024,
    BP_AT_PICKUPINVENTORY = 3025,
	
	//Only spawned actors that require a type
	BP_AT_MAP = 3100,
    BP_AT_FUNCTIONCALL = 3101,
    BP_AT_RAYTRACE = 3102,
	
	//Enemies
	BP_AT_TRICERATOPS = 3150,
	BP_AT_ROBOTGRUNT = 3151,
	BP_AT_ROBOTRIDER = 3152,
	
	//Bosses
    BP_AT_LONGHUNTER = 3200,
    BP_AT_HUMMER = 3201,
    BP_AT_MANTIS = 3202,
    BP_AT_TREX = 3203,
    BP_AT_CAMPAINGER = 3204,
    BP_AT_ROBOTLONGHUNTER = 3205
}

enum BPSystemFlags
{
    BP_SF_NOKNIFEINTERACTION = (1 << 0)
}

enum BPClipResultFlags
{
	CRF_NOCOLLISION = 0,
	CRF_FLOOR = (1 << 0), // made contact with a floor
	CRF_CEILING = (1 << 1), // made contact with a ceiling
	CRF_WALL = (1 << 2), // made contact with an edge (sector edge that isn't linked to another sector)
	CRF_OBJECT = (1 << 3), // made contact with an actor
	CRF_MESH = (1 << 4), // made contact with a static object
	CRF_ADJUST = (1 << 5), // z-axis was adjusted (ceiling or floor)
	CRF_WALLRADIUS = (1 << 6) // made contact with an edge during the wall radius collision test
}

enum BPEnumAIFlags
{
	//AIF_ATTACKING = (1 << 0)
	//AIF_WASSOLID = (1 << 1)
	AIF_FIRSTATTACK = (1 << 2),
	AIF_HEARDLOUDNOISE = (1 << 3),
	//AIF_BLOWNAWAY = (1 << 4)
	AIF_GOBACKTOLEASH = (1 << 5),
	AIF_RESURRECT = (1 << 6),
	AIF_REGENDELAY = (1 << 8),
	AIF_SEETARGET = (1 << 9),
	//AIF_NOCHASE = (1 << 10)
	AIF_REGENERATEANIM = (1 << 11),
	//AIF_NOTHINK = (1 << 12)
	//AIF_RUNNING = (1 << 13)
	AIF_GETATTENTION = (1 << 14),
	AIF_REGENDELAY2 = (1 << 15),
	AIF_HEARDQUIETNOISE = (1 << 16),
	AIF_AWAYFROMLEASH = (1 << 17),
	AIF_TELEPORTAWAY = (1 << 18),
	AIF_TELEPORTMOVESLOW = (1 << 19),
	//AIF_TELEPORTING = (1 << 20)
	AIF_WAITFORCYCLE = (1 << 22),
	AIF_DAMAGEPANIC = (1 << 23)
}

enum BPEnumAreaFlags
{
	// AAF_WATER = 1 << 0
	// AAF_BLOCK = 1 << 1
	// AAF_TOGGLE = 1 << 2
	// AAF_CLIFF = 1 << 3
	// AAF_CLIMB = 1 << 4
	// AAF_ONESIDED = 1 << 5 //Bridge
	// AAF_CEILING = 1 << 6
	// AAF_CRAWL = 1 << 7
	// AAF_ENTERCRAWL = 1 << 8
	// AAF_HIDDEN = 1 << 9
	// AAF_ENTERED = 1 << 10 //Not shown in Editor
	AAF_SECRET = 1 << 11,
	// AAF_RESTRICTED = 1 << 12
	// AAF_SLOPETEST = 1 << 13
	// AAF_DEATHPIT = 1 << 14
	// AAF_MAPPED = 1 << 15
	// AAF_EVENT = 1 << 16
	// AAF_REPEATABLE = 1 << 17
	// AAF_TELEPORT = 1 << 18
	// AAF_DAMAGE = 1 << 19
	// AAF_DRAWSKY = 1 << 20
	// AAF_TELEPORTAIR = 1 << 21
	// AAF_LAVA = 1 << 22
	// AAF_EVENTSOUND = 1 << 23
	// AAF_ANTIGRAVITY = 1 << 24
	// AAF_LADDER = 1 << 25
	// AAF_CHECKPOINT = 1 << 26
	// AAF_SAVEGAME = 1 << 27
	// AAF_WARPRETURN = 1 << 28
	// AAF_SHALLOWWATER = 1 << 29
	// AAF_DRAWSUN = 1 << 30
}
const uint AAF_STOREWARPRETURN = Math::UBIT31;

const int BP_AREAARG_WARP_TID = 0;
const int BP_AREAARG_WARP_MAPID = 1;
const int BP_AREAARG_CHECKPOINTID = 2;
const int BP_AREAARG_EVENTTID = 3;
const int BP_AREAARG_FLOORDAMAGE = 4;
const int BP_AREAARG_FLOORDAMAGERATE = 5; //1024 = 1 second

enum BPEnumImpactType
{
	//Already defined types
	// IT_DEFAULT
	// IT_WATER
	// IT_METAL
	// IT_STONE
	// IT_FLESH_HUMAN
	// IT_FLESH_CREATURE
	// IT_FLESH_UNDEAD
	// IT_LAVA
	// IT_SLIME
	// IT_FORCEFIELD
	
	//Custom Types
	IT_WOOD = 10
}

enum BPQuakeAnims
{
	BP_QUAKE_ANIM_NONE = 0,
	BP_QUAKE_ANIM_SHAKE = 1,
	BP_QUAKE_ANIM_THUMP = 2
}

kPuppet@ playerPuppet;
kActor@ playerActor;
//------------------------------------------------------------------------------------------------------------------------
void BP_Main()
{
	BP_ResetGameVars();
}
//------------------------------------------------------------------------------------------------------------------------
void BP_NewGame()
{
	BP_ResetGameVars();
}
//------------------------------------------------------------------------------------------------------------------------
void BP_MapStart()
{
	BP_MapStartSetup();
	Actor::Spawn(BP_AT_MAP, playerActor.Origin(), playerActor.Yaw(), playerActor.SectorIndex());
}
//------------------------------------------------------------------------------------------------------------------------
void BP_ResetGameVars()
{
	GameVar::SetInt(GV_BP_KEY_FLAGS, 0);
	GameVar::SetFloat(GV_BP_KEY_GAMETIME, 0.0f);
	GameVar::SetString(GV_BP_KEY_MUSIC, "");
	GameVar::SetInt(GV_BP_KEY_MUSICMAP, -2);
	GameVar::SetString(GV_BP_KEY_COUNTERS, "");
	Inventory::Reset();
}
//------------------------------------------------------------------------------------------------------------------------
// Everything in this function should be okay to be called twice
//------------------------------------------------------------------------------------------------------------------------
void BP_MapStartSetup()
{
	Game::LoadDef();
	Time::SetMapTime();
	@playerPuppet = Player.Actor();
	@playerActor = playerPuppet.CastToActor();
}
//------------------------------------------------------------------------------------------------------------------------
void Wait(const float time = 0.0f)
{
	delay(time);
}
//------------------------------------------------------------------------------------------------------------------------
void WaitUntil(WAITFUNC @func)
{
	if (func !is null) while (!func()) { Wait(); }
}
//------------------------------------------------------------------------------------------------------------------------
void WaitWhile(WAITFUNC @func)
{
	if (func !is null) while (func()) { Wait(); }
}
//------------------------------------------------------------------------------------------------------------------------
void DoUntil(DOFUNC@ doFunc, WAITFUNC@ waitFunc)
{
	if (doFunc is null || waitFunc is null) return;
	while (!waitFunc()) { doFunc(); Wait(); }
}
//------------------------------------------------------------------------------------------------------------------------
void DoWhile(DOFUNC@ doFunc, WAITFUNC@ waitFunc)
{
	if (doFunc is null || waitFunc is null) return;
	while (waitFunc()) { doFunc(); Wait(); }
}
//------------------------------------------------------------------------------------------------------------------------
void Log(const kStr &in s)
{
	Sys.Print(s);
}
//------------------------------------------------------------------------------------------------------------------------
void LogError(const kStr &in s)
{
	Sys.Warning(s);
}
//------------------------------------------------------------------------------------------------------------------------
void Assert(const bool condition, const kStr& in message)
{
	if (!condition)
	{
		LogError("[ASSERT] " + message);
		array<int> doCrash = { doCrash[0] };
	}
}
//------------------------------------------------------------------------------------------------------------------------
