//Should only be us in script module
namespace Cinema
{
	funcdef void CINEMAFINISHFUNC(const bool userInterrupted);

	float camLerpTime;
	float camLerpTotalTime;
	kVec3 camLerpFrom;
	kVec3 camLerpTo;
	float camShakeTime;
	float camShakePower;
	
	CINEMAFINISHFUNC@ finishedFunc;
	int scriptIDInstigator;
	//------------------------------------------------------------------------------------------------------------------------
	void StartScript(kActor@ instigator, const int scriptID, const bool waitUntilCinemaIsFinished = true)
	{
		Game.CallDelayedMapScript(scriptID, instigator, 0.0f);
		Wait(); //need a delay so the cinema script can be called before the rest of the current script is called
		if (waitUntilCinemaIsFinished)
			WaitUntil(IsFinished);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void Start(kActor@ instigator, const int scriptIDCaller, CINEMAFINISHFUNC@ endFunc = null, const uint cinFlags = CMF_LOCK_PLAYER|CMF_UNLOCK_PLAYER_ON_FINISH|CMF_NO_INITIAL_FADEOUT, const float fov = 74.0f)
	{
		scriptIDInstigator = scriptIDCaller;
		Game.HaltMapScript(SCRIPT_CINEMA_USER_INTERRUPT);
		@finishedFunc = endFunc;
		kActor@ pActor = Player.Actor().CastToActor();
		pActor.Flags() |= AF_NODRAW;
		pActor.AnimState().Stop();
		Camera.ClearLookAtActor();
		Camera.ClearFinalView();
		Camera.ClearViewTracks();
		Camera.StartCinematic(cinFlags);
		Camera.fov = fov;
		WaitUntil(IsPlaying);
		Game.CallDelayedMapScript(SCRIPT_CINEMA_USER_INTERRUPT, instigator, 0.0f);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void StartCustom(const uint cinFlags = CMF_NO_LETTERBOX|CMF_NO_INITIAL_FADEOUT, const float fov = 90.0f)
	{
		kActor@ pActor = Player.Actor().CastToActor();
		pActor.Flags() |= AF_NODRAW;
		pActor.AnimState().Stop();
		Camera.ClearLookAtActor();
		Camera.ClearFinalView();
		Camera.ClearViewTracks();
		Camera.StartCinematic(cinFlags);
		Camera.fov = fov;
		WaitUntil(IsPlaying);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void Stop(const bool userInterrupted = false)
	{
		if (userInterrupted)
		{
			if (scriptIDInstigator > 1)
				Game.HaltMapScript(scriptIDInstigator);
		}
		else
		{
			Game.HaltMapScript(SCRIPT_CINEMA_USER_INTERRUPT);
		}
		scriptIDInstigator = 0;
		Camera.StopCinematic();
		WaitUntil(IsFinished);
		CameraMoveStop();
		Camera.ClearLookAtActor();
		kActor@ pActor = Player.Actor().CastToActor();
		pActor.Flags() |= AF_NODRAW;
		pActor.AnimState().Stop();
		if (finishedFunc !is null)
		{
			finishedFunc(userInterrupted);
			@finishedFunc = null;
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
	void CameraMove(kActor@ instigator, const kVec3 &in from, const kVec3 &in to, const float time)
	{
		CameraMoveStop();
		camLerpFrom = from;
		camLerpTo = to;
		camLerpTotalTime = Math::Maxf(time, 0.001f);
		camLerpTime = 0.0f;
		Game.CallDelayedMapScript(SCRIPT_CINEMA_CAMERA_MOVE, instigator, 0.0f);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void CameraMoveStop()
	{
		Game.HaltMapScript(SCRIPT_CINEMA_CAMERA_MOVE);
		camLerpTime = camLerpTotalTime;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void CameraShake(kActor@ instigator, const float duration, const float power = 1.0f)
	{
		CameraShakeStop();
		camShakeTime = Time::FromNow(duration);
		camShakePower = power;
		Game.CallDelayedMapScript(SCRIPT_CINEMA_CAMERA_SHAKE, instigator, 0.0f);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void CameraShakeStop()
	{
		Game.HaltMapScript(SCRIPT_CINEMA_CAMERA_SHAKE);
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool IsCameraMoveFinished()
	{
		return camLerpTime >= camLerpTotalTime;
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool IsActive()
	{
		return Camera.Active();
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool IsPlaying()
	{
		return Camera.CinematicState() >= 1;
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool IsFinished()
	{
		return Camera.CinematicState() <= 0 || Camera.CinematicState() >= 5;
	}
	//------------------------------------------------------------------------------------------------------------------------
	// Shows player at the actorLocation
	//------------------------------------------------------------------------------------------------------------------------
	void ShowPlayer(kActor@ actorLocation, const int animID, const int modelVariation)
	{
		kActor@ actor = Player.Actor().CastToActor();
		actor.Flags() &= ~AF_NODRAW;
		actor.ModelVariation() = modelVariation;
		Player::SetPosition(actorLocation, actorLocation.Yaw());
		Player::PlayAnim(animID);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void LookTarget(kActor@ actor)
	{
		Camera.SetLookAtActor(actor);
	}
	//------------------------------------------------------------------------------------------------------------------------
}
