namespace Game
{
	array<kStr> fxNames;
	kActor@ raySrc;
	kActor@ rayDest;
	bool defLoaded;
	//------------------------------------------------------------------------------------------------------------------------
	// clipFlags: Specify what flags to not use (CF_NOCLIPACTORS means it will hit actors, 0 = everything)
	// Returns true if an intersection occured. CModel will store the result of the RayTrace.
	//------------------------------------------------------------------------------------------------------------------------
	bool RayTrace(const kVec3 &in from, const kVec3 &in to, uint clipFlags = 0)
	{
		if (!Actor::Exists(raySrc))
			@raySrc = Actor::Spawn(BP_AT_RAYTRACE, from);
		if (!Actor::Exists(rayDest))
			@rayDest = Actor::Spawn(BP_AT_RAYTRACE, to);
			
		raySrc.CanSee(rayDest, clipFlags);
		return CModel.Fraction() < 1.0f;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void AddShake(kActor @actor, const float duration, const bool thump = false)
	{
		AddShake(actor.Origin(), actor.SectorIndex(), duration, thump);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void AddShake(const kVec3 &in point, const int sectorID, const float duration, const bool thump = false)
	{
		kActor@ actor = Actor::Spawn(ACTOR_NAME_QUAKESOURCE, point, 0.0f, sectorID);
		actor.AnimState().Set(thump ? BP_QUAKE_ANIM_THUMP : BP_QUAKE_ANIM_SHAKE, 4.0f, 0);
		actor.StepHeight() = duration;
	}
	//------------------------------------------------------------------------------------------------------------------------
	int GetCounter(const int index)
	{
		return GameVar::GetIntFromStringArray(GV_BP_KEY_COUNTERS, index, MAX_COUNTERS);
	}
	//------------------------------------------------------------------------------------------------------------------------
	int IncreaseCounter(const int index, const int amount = 1)
	{
		return GameVar::AddIntToStringArray(GV_BP_KEY_COUNTERS, index, MAX_COUNTERS, amount);
	}
	//------------------------------------------------------------------------------------------------------------------------
	kStr GetFx(const int index)
	{
		if (HasFx(index - 1))
			return fxNames[index - 1];
		
		return "";
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool HasFx(const int index)
	{
		return Array::IndexInRange(fxNames.length(), index - 1);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void LoadDef()
	{
		if (!defLoaded)
		{
			defLoaded = true;
			kActor@ actor = Actor::Spawn(ACTOR_NAME_GAMEDEF);
			if (Actor::Exists(actor))
			{
				String::asciiChars[35] = Actor::GetDefString(actor, DEF_STRING_HASH);
				String::asciiChars[36] = Actor::GetDefString(actor, DEF_STRING_DOLLAR);
				fxNames = Actor::GetDefStringArray(actor, "fx");
				
				actor.Remove();
			}
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
}
