namespace GameVar
{
	//------------------------------------------------------------------------------------------------------------------------
	// Returns true if added the key to GameVariables
	//------------------------------------------------------------------------------------------------------------------------
	bool HasKey(const kStr &in key)
	{
		return GameVariables.HasKey(key);
	}
	//------------------------------------------------------------------------------------------------------------------------
	float GetFloat(const kStr &in key, const float defaultValue = 0.0f)
	{
		float value;
		GameVariables.GetFloat(key, value, defaultValue);
		return value;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void SetFloat(const kStr &in key, const float value)
	{
		SetValue(key, "" + value);
	}
	//------------------------------------------------------------------------------------------------------------------------
	int GetInt(const kStr &in key, const int defaultValue = 0)
	{
		int value;
		GameVariables.GetInt(key, value, defaultValue);
		return value;
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool GetIntFlag(const kStr &in key, const int flag)
	{
		int value;
		GameVariables.GetInt(key, value);
		return (value & flag != 0);
	}
	//------------------------------------------------------------------------------------------------------------------------
	void SetInt(const kStr &in key, const int value)
	{
		SetValue(key, "" + value);
	}
	//------------------------------------------------------------------------------------------------------------------------
	// Returns the keys value
	//------------------------------------------------------------------------------------------------------------------------
	int SetIntFlag(const kStr &in key, const int flags, const bool on = true)
	{
		int value = GetInt(key);
		if (on)
			value |= flags;
		else
			value &= ~flags;
		SetValue(key, "" + value);
		
		return value;
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool GetBool(const kStr &in key, const bool defaultValue = false)
	{
		bool value;
		GameVariables.GetBool(key, value, defaultValue);
		return value;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void SetBool(const kStr &in key, const bool value)
	{
		SetValue(key, value ? "1" : "0");
	}
	//------------------------------------------------------------------------------------------------------------------------
	kVec3 GetVector(const kStr &in key)
	{
		kVec3 value;
		GameVariables.GetVector(key, value);
		return value;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void SetVector(const kStr &in key, kVec3 &in value)
	{
		SetValue(key, value.ToString());
	}
	//------------------------------------------------------------------------------------------------------------------------
	kStr GetString(const kStr &in key)
	{
		kStr value;
		GameVariables.GetString(key, value);
		return value;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void SetString(const kStr &in key, const kStr &in value)
	{
		SetValue(key, value);
	}
	//------------------------------------------------------------------------------------------------------------------------
	int GetIntFromStringArray(const kStr &in key, const int index, const int aryLength)
	{
		if (!Array::IndexInRange(aryLength, index))
			return 0;
		
		return GetIntArrayFromString(key, aryLength)[index];
	}
	//------------------------------------------------------------------------------------------------------------------------
	int AddIntToStringArray(const kStr &in key, const int index, const int aryLength, const int amount)
	{
		if (!Array::IndexInRange(aryLength, index))
			return 0;

		array<int> ary = GetIntArrayFromString(key, aryLength);
		ary[index] += amount;
		SetIntArrayToString(key, ary);
		
		return ary[index];
	}
	//------------------------------------------------------------------------------------------------------------------------
	array<int> GetIntArrayFromString(const kStr &in key, const int aryLength)
	{
		array<int> ary = String::SplitInt(GetString(key), " ");
		if (int(ary.length()) != aryLength)
			ary.resize(aryLength);
		
		return ary;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void SetIntArrayToString(const kStr &in key, array<int> &ary)
	{
		SetString(key, Array::ToString(ary));
	}
	//------------------------------------------------------------------------------------------------------------------------
	void SetValue(const kStr &in key, const kStr &in value)
	{
		if (HasKey(key))
		{
			GameVariables.SetValue(key, value);
		}
		else
		{
			GameVariables.Add(key, value);
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
}
