#include "scripts/BP/JH_Common_Map.cpp"

enum EnumMapTags
{
	TAG_PLAYER_CINEMA_START = 1000,
	TAG_PLAYER_RIDING_RESET_POINT = 1001,
	TAG_RIDING_START_POINT = 1002,
}

//TIDs
//1000: Player Opening Cinema Start Point
//1001: Player Riding Bird Reset Point
//1002: Riding Bird Start Point
//1003: 2 Health Spawner
//1004: Circle Tree Spawner
//1005-1007: Poly Tree Spawner
//1008: 
//1009: 
//1010-1023: Poly Tree Spawner

//2000-2049: Grunt Generic Axe (14hp)
//2050-2099: Grunt Generic Club (14hp)
//2100-2149: Grunt Commander Spear (14hp)
//2150-2199: Grunt Commander Rifle2 (9hp)
//2200-2249: Grunt Commander Rifle4 (75hp)
//2250-2299: Poacher Knife (14hp)
//2300-2349: Poacher Pistol1 (14hp)
//2350-2399: Poacher Rifle (9hp)
//2400-2449: Poacher Shotgun (9hp)
//2450-2499: Warrior Bone (28hp)
//2500-2549: Warrior Dart (28hp)
//2550-2599: Warrior Spear (28hp)
//2600-2649: Shaman (150hp)
//2650-2699: Cyborg Pulse Rifle (150hp)
//2700-2749: Cyborg Knife (30hp)
//2750-2799: Cyborg Spear (30hp)
//2800-2849: Demon Axe (56hp)
//2850-2899: Demon Spear (56hp)
//2900-2949: Demon Hands (35hp)
//2950-2999: Demon Lord (300hp)
//3000-3049: DragonFly (1hp)
//3050-3099: Raptor Tech (125hp)
//3100-3149: Triceratops Rider (500hp)
//3150-3199: Dimetrodon Tech (200hp)
//3200-3249: Purlin Tech (150hp)
//3250-3299: Sandworm (125hp)
//3300-3349: Alien (60hp)
//3350-3399: Mech Red (150hp)
//3400-3449: Mech Orange (250hp)
//3450-3499: Mech Turret (150hp)
//3500-3549: Killer Plant (100hp)
//3550-3599: Purlin (150hp)
//3600-3649: Dimetrodon (200hp)
//3650-3699: Leaper (10hp)
//3700-3749: Raptor (50hp)
//3750-3799: Trex Boss
//3800-3849: Mantis Boss
//3850-3899: Longhunter Boss
//3900-3949: Hummer Boss
//3950-3999: Campaigner Boss

//10000: Start Warp Point
//10001: Save 1 Warp Point


//max map vert position: 14346.239258
//water height +8.0
//ob level scaled by 35 (from game to in blender) then 256 (in unity)
//In Editor Pos Offset: 1177.599976 5314.560059 0.000000

const int TAG_ENEMIES_FIRST = 2000;
const int TAG_ENEMIES_LAST = 4000;

const int SCRIPT_OPENING_CINEMA = 2;

enum FlyingDinoAnims
{
    FDAnim_Idle = 0,
    FDAnim_Flap,
    FDAnim_Dive
}

bool isRidingBird;
float birdMaxPitch = Math::Deg2Rad(89.0f); //89.0f or lower
float birdPitchScale = 2.5f;
float birdMaxTurnSpeed = Math::Deg2Rad(7.5f);
float birdMaxTurnSpeedScale = 2.0f;
float birdFastForwardSpeed = 30.0f;
float birdForwardSpeed = 14.0f;
float birdSlowForwardSpeed = 7.0f;
float lastPlayerYaw;
float lastPlayerPitch;
float tempDelay; //delete
kVec3 ridingPlayerPos;
int ridingPlayerSector;
float birdAttackTime;
int birdCurrentWeapon;

kActor@ bird;
kActor@ birdLook;
kActor@ birdReticle;
kActor@ birdStartPoint;
kActor@ birdFirePoint;
kActor@ playerBirdResetPoint;

BP_ActorIterator aiIter;

//------------------------------------------------------------------------------------------------------------------------
// OnStart
//------------------------------------------------------------------------------------------------------------------------
$script 0
{
	OnMapStart(instigator);

	aiIter.Setup(2, TickAI);
	for (int i = TAG_ENEMIES_FIRST; i <= TAG_ENEMIES_LAST; i ++)
	{
		kActor@ actor = World.GetActorByTID(i);
		if (Actor::Exists(actor))
		{
			aiIter.AddActor(actor);
		}
	}
	
	@playerBirdResetPoint = World.GetActorByTID(TAG_PLAYER_RIDING_RESET_POINT);
	@birdStartPoint = World.GetActorByTID(TAG_RIDING_START_POINT);
	@bird = Actor::Spawn(ACTOR_NAME_FLYINGRIDE, birdStartPoint.Origin().x, birdStartPoint.Origin().y, birdStartPoint.Origin().z, birdStartPoint.Yaw(), birdStartPoint.SectorIndex());
	@birdFirePoint = Actor::Spawn(ACTOR_NAME_DUMMY, birdStartPoint.Origin().x, birdStartPoint.Origin().y, birdStartPoint.Origin().z, birdStartPoint.Yaw(), birdStartPoint.SectorIndex());
	@birdReticle = Actor::Spawn(ACTOR_NAME_DUMMY, birdStartPoint.Origin().x, birdStartPoint.Origin().y, birdStartPoint.Origin().z, birdStartPoint.Yaw(), birdStartPoint.SectorIndex());
	bird.Flags() &= ~AF_SOLID;
	bird.Flags() |= AF_ALWAYSACTIVE;
	bird.Flags() |= AF_NODRAW;
	bird.AnimState().Set(FDAnim_Flap, 4.0f, ANF_LOOP);
	bird.ClipFlags() |= CF_CLIPEDGES | CF_DROPOFF | CF_WALKWALLS | CF_COLLIDEFLOORS | CF_NOSLOPESTEP | CF_COLLIDEHEIGHT | CF_NOFLOORADJUST | CF_NOCEILINGADJUST | CF_COLLIDEWATER;
	bird.RenderModel().SetRotationOffset(0, Math::Deg2Rad(180.0f), 0.0f, 0.0f, 1.0f);

	@birdLook = Actor::Spawn(ACTOR_NAME_DUMMY, 0.0f, 0.0f, 0.0f, 0.0f, -1);
	birdLook.Flags() &= ~AF_SOLID;

	if (isNewGame)
	{
		//force player to pickup invisible newgame key and prevent the pickup key cinematic and text from showing
		Cinema::StartCustom(); //so user sees cinema as the first thing
		kActor@ startPoint = World.GetActorByTID(TAG_PLAYER_CINEMA_START);
		Player::SetPosition(startPoint, startPoint.Yaw());
		playerActor.Velocity() = kVec3(0.0f, 0.0f, 0.01f);
		Wait();
		Camera.StopCinematic();
		WaitUntil(Cinema::IsFinished);
		HUD::ClearMessages();

		Cinema::StartScript(instigator, SCRIPT_OPENING_CINEMA);
		Music::Play(PATH_MUSIC_JUNGLEHUNTER);
	}
	else
	{
		Music::Play(PATH_MUSIC_INTOTHEJUNGLE);
	}
	
}
//------------------------------------------------------------------------------------------------------------------------
// OnTick
//------------------------------------------------------------------------------------------------------------------------
$script 1
{
	OnMapTick();
	aiIter.Tick();
    RidingBirdTick();
	
	if (!Actor::IsDead(playerActor))
	{
		if (Time::Reached(tempDelay) && Input::Pressing(OPEN_MENU_BUTTONS))
		{
			tempDelay = Time::FromNow(0.5f);
			
			GameVar::SetIntFlag(GV_BP_KEY_FLAGS, BP_SF_NOKNIFEINTERACTION, !isRidingBird);
			
			if (isRidingBird)
			{
				RidingBirdFinish();
			}
			else
			{
				RidingBirdStart();
			}
		}
	}

    $restart;
}
//------------------------------------------------------------------------------------------------------------------------
// Play Opening Cinema
//------------------------------------------------------------------------------------------------------------------------
$script 2
{
	Cinema::Start(instigator, SCRIPT_OPENING_CINEMA, OnOpeningCinemaEnd);
	Cinema::ShowPlayer(playerActor, anim_player_idle, TV_BOW);
	Cinema::LookTarget(playerActor);
	Cinema::CameraMove(instigator, kVec3(-12587, -4753, 273), kVec3(-12731, -3875, 929), 3.0f);
	WaitUntil(Cinema::IsCameraMoveFinished);
	Wait(1.0f);
	Cinema::Stop();
}
//------------------------------------------------------------------------------------------------------------------------
void OnOpeningCinemaEnd(const bool userInterrupted)
{
	kActor@ playerEndPoint = World.GetActorByTID(TAG_PLAYER_CINEMA_START);
	Player::SetPosition(playerEndPoint, Math::Deg2Rad(-10.0f), Math::Deg2Rad(-25.0f));
}
//------------------------------------------------------------------------------------------------------------------------
bool TickAI(kActor@ actor)
{
	bool exists = Actor::Exists(actor);
	if (exists)
	{
		actor.MarkPersistentBit(true);
	}
	
	return !exists;
}
//------------------------------------------------------------------------------------------------------------------------
void RidingBirdStart()
{
	isRidingBird = true;

	ridingPlayerPos = playerActor.Origin();
	ridingPlayerSector = playerActor.SectorIndex();
	playerActor.Origin() = playerBirdResetPoint.Origin();
	playerActor.SetSector(playerBirdResetPoint.SectorIndex());
	playerPuppet.PlayerFlags() |= PF_GOD;
	
	Wait();
	Cinema::StartCustom();
	Cinema::LookTarget(birdLook);
	kVec3 birdForwardDir = Actor::ForwardDir(bird);
	Camera.origin = bird.Origin() - (birdForwardDir * 600.0f) + kVec3(0,0,350);

	bird.Flags() &= ~AF_NODRAW;

}
//------------------------------------------------------------------------------------------------------------------------
void RidingBirdFinish()
{
	playerActor.Origin() = ridingPlayerPos;
	playerActor.SetSector(ridingPlayerSector);
	
	Wait();
	
	isRidingBird = false;
	Cinema::Stop();
	playerPuppet.PlayerFlags() &= ~PF_GOD;
	bird.Velocity() = kVec3(0.0f, 0.0f, 0.0f);

	//bird.Flags() |= AF_NODRAW;
	//bird.Flags() &= ~AF_SOLID;
}
//------------------------------------------------------------------------------------------------------------------------
void RidingBirdTick()
{
	if (!isRidingBird)
		return;

	playerActor.Origin() = playerBirdResetPoint.Origin();
	if (PlayLoop.Ticks() % 2 == 0)
		playerActor.Origin().z -= 300.0f;
	//Prevent player jumping
	playerPuppet.PlayerFlags() |= PF_HASJUMPED;

	kVec3 birdForwardDir = Actor::ForwardDir(bird);

	//birdLook.SpawnFx("fx/spellcast.kfx", kVec3(0, 0, 0));

	// if (PlayLoop.Ticks() % 40 == 0)
	// {
		//Log("Dir: " + birdForwardDir.ToString() + " Vel: " + bird.Velocity().ToString());
	// }

	//SetCamera Position behind the player
	
	//Camera.origin = bird.Origin() - (birdForwardDir * 600.0f) + kVec3(0,0,350);
	
	Camera.origin.Lerp(bird.Origin() - (birdForwardDir * 450.0f) + kVec3(0,0,175), 0.1f);
	
	//birdLook.Origin() = bird.Origin() + (birdForwardDir * 600.0f);
	birdLook.Origin().Lerp(bird.Origin() + (birdForwardDir * 600.0f), 0.1f);
	
	//Camera.yaw = playerActor.Yaw();

	float speed = birdForwardSpeed;
	if (Input::ForwardPress())
		speed = birdFastForwardSpeed;
	else if (Input::BackwardPress())
		speed = birdSlowForwardSpeed;

	//Set Bird Velocity Forward
	bird.Velocity() = birdForwardDir * speed;

	//Slowly rotate bird towards player yaw/pitch
	float diffYawAngle = Math::Clampf(playerActor.Yaw().Diff(lastPlayerYaw), -birdMaxTurnSpeed, birdMaxTurnSpeed) * birdMaxTurnSpeedScale;
	lastPlayerYaw = playerActor.Yaw();
	bird.Yaw() = bird.Yaw().Interpolate(bird.Yaw() + diffYawAngle, 0.25f);
	//bird.Yaw() = playerActor.Yaw();
	//* birdPitchSacle
	float diffPitchAngle = playerActor.Pitch().Diff(lastPlayerPitch) * birdPitchScale;
	//float targetPitch = Math::Clampf(playerActor.Pitch(), -birdMaxPitch, birdMaxPitch);
	lastPlayerPitch = playerActor.Pitch();
	bird.Pitch() = bird.Pitch().Interpolate(bird.Pitch() + diffPitchAngle, 0.25f);
	bird.Pitch() = Math::Clampf(bird.Pitch(), -birdMaxPitch, birdMaxPitch);
	//bird.Pitch() = bird.Pitch().Interpolate(targetPitch, 0.01f);

	//birdReticle.Yaw() = bird.Yaw();
	//birdReticle.Pitch() = bird.Pitch();
	//birdReticle.Origin() = bird.Origin() + (Actor::ForwardDir(bird) * 600.0f);
	birdReticle.Origin().Lerp(bird.Origin() + (birdForwardDir * 600.0f), 0.25f);
	birdReticle.SpawnFx("fx/BP/bird_target_reticle.kfx", kVec3(0, 0, 0));


	
	bird.SetTarget(birdLook);
	
	birdFirePoint.Origin() = bird.Origin();
	birdFirePoint.Yaw() = Camera.yaw;
	birdFirePoint.Pitch() = Camera.pitch;
	birdFirePoint.Roll() = Camera.roll;
	birdFirePoint.SetTarget(birdReticle);
	
	if (Input::AttackPress() && Time::Reached(birdAttackTime))
	{
		//fire at target
		//if (bird.GetTarget() !is null)
		//{
			//bird.SpawnProjectile("fx/tech_explosion.kfx", kVec3(0.0f, 0.0f, 0.0f), birdLook.Origin(), Math::Deg2Rad(45.0f));
			if (birdCurrentWeapon == 0)
			{
				
				birdAttackTime = Time::FromNow(0.3f);
				//bird.SpawnFx("fx/BP/bird_nuke.kfx", kVec3(0, 0, 0));
				birdFirePoint.SpawnFx("fx/BP/bird_nuke.kfx", kVec3(0, 0, 0));
				//bird.SpawnProjectile("fx/BP/bird_nuke.kfx", kVec3(0.0f, 0.0f, 0.0f), birdLook.Origin(), Math::Deg2Rad(45.0f));
			}
			else
			{
				birdAttackTime = Time::FromNow(0.1f);
				//bird.SpawnProjectile("fx/BP/bird_rocket.kfx", kVec3(0.0f, 0.0f, 0.0f), birdLook.Origin(), Math::Deg2Rad(45.0f));
				//birdFirePoint.SpawnProjectile("fx/BP/bird_rocket.kfx", kVec3(0.0f, 0.0f, 0.0f), birdLook.Origin(), Math::Deg2Rad(45.0f));
				birdFirePoint.SpawnFx("fx/BP/bird_rocket.kfx", kVec3(0, 0, 0));
			}
			//bird.SpawnFx("fx/spellcast.kfx", kVec3(0, 0, 0));
		//}
	}
		
	if (Input::MapZoomInDown())
	{
		birdCurrentWeapon++;
		if (birdCurrentWeapon > 1)
			birdCurrentWeapon = 0;
	}
	
	// if (Input::StrafeLeftPress())
	// {
		// //move bird to the right of the camera look target and rotate 90 degrees, make turning left faster
	// }
	// else if (Input::StrafeRightPress())
	// {
		// //move bird to the left of the camera look target and rotate 90 degrees, make turning right faster
	// }
}
//------------------------------------------------------------------------------------------------------------------------
